Relations

Struct Relations 

pub struct Relations {
Show 24 fields pub symbols: Vec<SymbolFact>, pub imports: Vec<ImportFact>, pub calls: Vec<CallFact>, pub visibilities: Vec<VisibilityFact>, pub attributes: Vec<AttributeFact>, pub parents: Vec<ParentFact>, pub qualifiers: Vec<QualifierFact>, pub symbol_ranges: Vec<SymbolRangeFact>, pub implements: Vec<ImplementsFact>, pub is_impls: Vec<IsImplFact>, pub type_methods: Vec<TypeMethodFact>, pub resolved_imports: Vec<ResolvedImportFact>, pub modules: Vec<ModuleFact>, pub exports: Vec<ExportFact>, pub reexports: Vec<ReexportFact>, pub symbol_uses: Vec<SymbolUseFact>, pub resolved_references: Vec<ResolvedReferenceFact>, pub resolved_calls: Vec<ResolvedCallFact>, pub module_search_paths: Vec<ModuleSearchPathFact>, pub cfg_blocks: Vec<CfgBlockFact>, pub cfg_edges: Vec<CfgEdgeFact>, pub cfg_defs: Vec<CfgDefFact>, pub cfg_uses: Vec<CfgUseFact>, pub cfg_effects: Vec<CfgEffectFact>,
}
All relations (facts) available to rules.

This is the complete set of facts extracted from a codebase. Rule packs receive this and apply Datalog rules over it.

Fields

symbols: Vec<SymbolFact>

All symbols defined in the codebase

imports: Vec<ImportFact>

All imports in the codebase

calls: Vec<CallFact>

All function calls in the codebase

visibilities: Vec<VisibilityFact>

Symbol visibility facts

attributes: Vec<AttributeFact>

Symbol attribute facts (one per attribute per symbol)

parents: Vec<ParentFact>

Symbol parent-child hierarchy

qualifiers: Vec<QualifierFact>

Call qualifier facts (receiver/module on calls)

symbol_ranges: Vec<SymbolRangeFact>

Symbol range facts (start and end lines)

implements: Vec<ImplementsFact>

Implements facts (symbol implements interface/trait)

is_impls: Vec<IsImplFact>

Is-impl facts (symbol is a trait/interface implementation)

type_methods: Vec<TypeMethodFact>

Type method facts (method signatures on types)

resolved_imports: Vec<ResolvedImportFact>

Resolved import facts (import resolved to a specific file)

modules: Vec<ModuleFact>

Module identity facts (file → canonical module path)

exports: Vec<ExportFact>

Export facts (symbol exported from a file)

reexports: Vec<ReexportFact>

Reexport facts (symbol re-exported through an intermediate)

symbol_uses: Vec<SymbolUseFact>

Symbol use facts (reference/use sites)

resolved_references: Vec<ResolvedReferenceFact>

Resolved reference facts (use resolved to definition)

resolved_calls: Vec<ResolvedCallFact>

Resolved call facts (call resolved to definition file)

module_search_paths: Vec<ModuleSearchPathFact>

Module search path facts (directories to search for modules)

cfg_blocks: Vec<CfgBlockFact>

CFG block facts (one per basic block per function)

cfg_edges: Vec<CfgEdgeFact>

CFG edge facts (one per control-flow edge per function)

cfg_defs: Vec<CfgDefFact>

CFG variable definition facts (one per def site per block)

cfg_uses: Vec<CfgUseFact>

CFG variable use facts (one per use site per block)

cfg_effects: Vec<CfgEffectFact>

CFG side-effect facts (await, defer, yield, acquire, release, send, receive)

Implementations

Source

impl Relations

pub fn new() -> Self

Create empty relations

pub fn add_symbol(&mut self, file: &str, name: &str, kind: &str, line: u32)

Add a symbol fact

pub fn add_import(&mut self, from_file: &str, to_module: &str, name: &str)

Add an import fact

pub fn add_call( &mut self, caller_file: &str, caller_name: &str, callee_name: &str, line: u32, )

Add a call fact

pub fn add_visibility(&mut self, file: &str, name: &str, visibility: &str)

Add a visibility fact

pub fn add_attribute(&mut self, file: &str, name: &str, attribute: &str)

Add an attribute fact

pub fn add_parent(&mut self, file: &str, child_name: &str, parent_name: &str)

Add a parent fact

pub fn add_qualifier( &mut self, caller_file: &str, caller_name: &str, callee_name: &str, qualifier: &str, )

Add a qualifier fact

pub fn add_symbol_range( &mut self, file: &str, name: &str, start_line: u32, end_line: u32, )

Add a symbol range fact

pub fn add_implements(&mut self, file: &str, name: &str, interface: &str)

Add an implements fact

pub fn add_is_impl(&mut self, file: &str, name: &str)

Add an is_impl fact

pub fn add_type_method( &mut self, file: &str, type_name: &str, method_name: &str, )

Add a type method fact

pub fn add_resolved_import( &mut self, from_file: &str, to_file: &str, imported_name: &str, local_alias: &str, kind: &str, )

Add a resolved import fact

pub fn add_module(&mut self, file: &str, canonical_module_path: &str)

Add a module fact

pub fn add_export(&mut self, file: &str, name: &str, kind: &str)

Add an export fact

pub fn add_reexport( &mut self, from_file: &str, original_file: &str, original_name: &str, exported_as: &str, )

Add a reexport fact

pub fn add_symbol_use(&mut self, file: &str, name: &str, line: u32)

Add a symbol use fact

pub fn add_resolved_reference( &mut self, use_file: &str, use_line: u32, def_file: &str, def_name: &str, def_kind: &str, )

Add a resolved reference fact

pub fn add_resolved_call( &mut self, caller_file: &str, caller_name: &str, callee_file: &str, callee_name: &str, line: u32, )

Add a resolved call fact

pub fn add_module_search_path( &mut self, workspace_root: &str, language: &str, kind: &str, path: &str, )

Add a module search path fact

pub fn add_cfg_block( &mut self, file: &str, func: &str, func_line: u32, block: u32, kind: &str, )

Add a CFG block fact

pub fn add_cfg_edge( &mut self, file: &str, func: &str, func_line: u32, from: u32, to: u32, kind: &str, exception_type: &str, )

Add a CFG edge fact

pub fn add_cfg_def( &mut self, file: &str, func: &str, func_line: u32, block: u32, name: &str, )

Add a CFG variable definition fact

pub fn add_cfg_use( &mut self, file: &str, func: &str, func_line: u32, block: u32, name: &str, )

Add a CFG variable use fact

pub fn add_cfg_effect( &mut self, file: &str, func: &str, func_line: u32, block: u32, kind: &str, line: u32, label: &str, )

Add a CFG side-effect fact